#include <IRremote.h> // IR Bibliothek einbinden
#include <SoftwareSerial.h> // Software Serial Bibliothek einbinden
#include <TimerFreeTone.h>

// Pins deklarieren

const int BTRX=2; // Arduino Recieve <- BT Send
const int BTTX=3; // Arduino Send -> BT Receive
const int motor1a=5;
const int motor1b=4;
const int motor1speed=6; // muss PWM Pin sein
const int lichtpin=7;
const int hupe=8;
const int statusled=9;
const int taster=10;
const int irpin=11;
const int standby=12;
// Variablen deklarieren

int richtung=0;
boolean leuchten=false;
int gang=3; // Startgeschwindigkeit 150
int geschwindigkeit[6] = {0,50,100,150,200,255};
int tasterstatus=0;

SoftwareSerial SerialBT(BTRX, BTTX);

int msg; // BT Befehl

IRrecv irrecv(irpin); // Code für IR
decode_results results; // Code für IR





//////////////////////// Fahrfunktionen ///////////////////////////


// vorwärts
void vor()
{
if (richtung==0)
  { 
  analogWrite(motor1speed,0); // von welcher geschwindigkeit aus beschleunigt wird
  digitalWrite(motor1a, HIGH);
  digitalWrite(motor1b, LOW);
  richtung=1;
  
  beschleunigenvor();
  }
if (richtung==-1)  
  {
  
  bremsen();
  digitalWrite(motor1a, HIGH);
  digitalWrite(motor1b, LOW);
  richtung=1;
  
  beschleunigenvor();
  }
}

// rückwärts fahren
void zurueck()
{
if (richtung==0)
  { 
  analogWrite(motor1speed,0); // von welcher geschwindigkeit aus beschleunigt wird   
  digitalWrite(motor1a, LOW);
  digitalWrite(motor1b, HIGH);
  richtung=-1;
  
  beschleunigenzurueck();
  }
if (richtung==1)
  {
  
  bremsen();
  digitalWrite(motor1a, LOW);
  digitalWrite(motor1b, HIGH);    
  richtung=-1;
  
  beschleunigenzurueck();
  }
}

// stopp
void stopp()
{
if (richtung!=0)
  {
  
  bremsen();
  digitalWrite(motor1a, LOW);
  digitalWrite(motor1b, LOW);
  richtung=0;
  }
}

// Geschwindigkeit erhöhen

void schneller()
{
if (gang<5)
  {
  gang++;
  analogWrite(motor1speed, geschwindigkeit[gang]);
  delay(30); // um nur einen gang hochzuschalten
  for (int i=0; i<gang; i++)
    {
      blinken();
    }
  }
}

// Geschwindigkeit verringern
void langsamer()
{
if (gang>1)
  {
  gang--;
  analogWrite(motor1speed, geschwindigkeit[gang]);
  delay(30); // um nur einen gang runterzuschalten
  for (int i=0; i<gang; i++)
    {
      blinken();
    } 
  }
}


// Hupen
void hupen()
{
TimerFreeTone(8, 200,180);
         delay (150);
         TimerFreeTone(8, 200,350);
}   

// Licht toggeln
void licht()
{
  if (leuchten == false)  // wenn Licht aus ...
    {
      digitalWrite(lichtpin, HIGH); // ...dann anschalten...
      leuchten = true;
      SerialBT.println("Licht an"); // Ausgabe auf der seriellen Schnittstelle
       Serial.println("Licht an"); // Ausgabe auf der seriellen Schnittstelle
      delay(30);
    } 
  else 
    {
      digitalWrite(lichtpin, LOW); // ...sonst ausschalten
      leuchten = false;
      SerialBT.println("Licht aus"); // Ausgabe auf der seriellen Schnittstelle
      Serial.println("Licht aus"); // Ausgabe auf der seriellen Schnittstelle
      delay(30);
    }
}

// Statusled Blink
void blinken()
{
  digitalWrite(statusled, LOW);
  delay(70);
  digitalWrite(statusled, HIGH);
  delay(70);
}

// bremsen bis zum Stillstand
void bremsen()
{
  SerialBT.println("bremsen"); // Ausgabe auf der seriellen Schnittstelle
      for (int aktuellegeschwindigkeit=geschwindigkeit[gang]; aktuellegeschwindigkeit>0; aktuellegeschwindigkeit--) // solange Geschwindigkeit da ist, diese verringern
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(2); // beeinflusst die Bremszeit
    }  
}

// beschleunigen vorwärts aus stand
void beschleunigenvor()
{
  SerialBT.println("beschleunigen vorwärts"); // Ausgabe auf der seriellen Schnittstelle
     for (int aktuellegeschwindigkeit=0; aktuellegeschwindigkeit<geschwindigkeit[gang]; aktuellegeschwindigkeit++) // solange beschleunigen, bis eingestellte Geschwindigkeit erreicht ist
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(7); // beeinflusst die Beschleunigungszeit
    }
}

// beschleunigen rückwärts
void beschleunigenzurueck()
{
  SerialBT.println("beschleunigen rückwärts"); // Ausgabe auf der seriellen Schnittstelle
     for (int aktuellegeschwindigkeit=0; aktuellegeschwindigkeit<geschwindigkeit[gang]; aktuellegeschwindigkeit++) // solange beschleunigen, bis eingestellte Geschwindigkeit erreicht ist
    {
      analogWrite(motor1speed, aktuellegeschwindigkeit); // Geschwindigkeit setzen
      delay(7); // beeinflusst die Beschleunigungszeit
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Setup //////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void setup() {
   
SerialBT.begin(9600); // Startet die Datenübertragung mit 9600 Baud
SerialBT.println("Verbindung steht");

Serial.begin(9600); // Startet die Datenübertragung mit 9600 Baud
Serial.println("Verbindung steht");

irrecv.enableIRIn(); // Code für IR

pinMode(irpin,INPUT);
pinMode(taster, INPUT); // Taster für fahren

pinMode(motor1a,OUTPUT); // Motor 1a
pinMode(motor1b,OUTPUT); // Motor 1b
pinMode(motor1speed,OUTPUT); // Motor 1c - PWM Pin
pinMode(standby,OUTPUT); // Schalter für Motortreiber
pinMode(lichtpin,OUTPUT); // Licht
pinMode(hupe,OUTPUT); // Hupe
pinMode(statusled,OUTPUT); // Status LED


// alles auschalten

digitalWrite(standby,HIGH); // Motortreiber aktivieren
digitalWrite(motor1a,LOW); // Motor 1a aus
digitalWrite(motor1b,LOW); // Motor 1b aus
analogWrite(motor1speed,geschwindigkeit[gang]); // Motor 1 startup Geschwindigkeit
digitalWrite(lichtpin,LOW); // Licht aus
digitalWrite(statusled,HIGH); // StatusLED an

}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// LOOP ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void loop() {

// Taster abfragen

tasterstatus = digitalRead(taster);
if (tasterstatus == HIGH)
{
  if (richtung==0)
  {
  SerialBT.println("vorwärts"); // Ausgabe auf der Seriellen schnittstelle
  gang=5;
  vor();
  blinken();
  }
  else
  {
  SerialBT.println("stopp"); // Ausgabe auf der Seriellen schnittstelle
  stopp();
  blinken();
  }
}


    
if (SerialBT.available() > 0) // wenn sinnvolle BT Daten empfangen werden
{      
     msg = SerialBT.read(); // Nachricht lesen

switch(msg) // Abhaengig vom Signal in den richtigen Modus springen
{
 case 48: // 0 Vorwärts
      SerialBT.println("vorwärts"); // Ausgabe auf der Seriellen schnittstelle
      vor();
      blinken();
 break;
 
 case 49: // 1 Halt
      SerialBT.println("stopp"); // Ausgabe auf der Seriellen schnittstelle
      stopp();
      blinken();
 break;

 case 50: // 2 Rückwärts
      SerialBT.println("rückwärts"); // Ausgabe auf der Seriellen schnittstelle
      zurueck();
      blinken();
 break;

 case 51: // 3 langsamer
      SerialBT.println("langsamer"); // Ausgabe auf der Seriellen schnittstelle
      langsamer();
      SerialBT.println(gang); // Ausgabe auf der Seriellen schnittstelle
      blinken();
 break;    

 case 52: // 4 schneller
      SerialBT.println("schneller"); // Ausgabe auf der Seriellen schnittstelle
      schneller();
      SerialBT.println(gang); // Ausgabe auf der Seriellen schnittstelle
      blinken();
 break;

 case 53: // 5 Licht schalten
      licht();
      blinken();
 break;

 case 54: // 6 Hupen
      SerialBT.println("Hupen"); // Ausgabe auf der Seriellen schnittstelle
      hupen();
      blinken();
 break;


}
}

// IR Fernbedinung Programm

 if (irrecv.decode(&results)) 
 {
 irrecv.resume(); // Das Signal empfangen
 }

 switch(results.value) // Abhaengig vom Signal in den richtigen Modus springen
 {
 
 case 16718055:// Vorwaerts
      SerialBT.println("vorwärts"); // Ausgabe auf der Seriellen schnittstelle
      vor();
      blinken();
 break;
 
 case 16726215: // Stopp
      SerialBT.println("Stopp"); // Ausgabe auf der Seriellen schnittstelle
      stopp();
      blinken();
 break;
 
 case 16730805: // Rueckwaerts
      SerialBT.println("Rückwärts"); // Ausgabe auf der Seriellen schnittstelle
      zurueck();
      blinken();
 break;
 
 case 16716015: // langsamer
      SerialBT.println("lansagmer"); // Ausgabe auf der Seriellen schnittstelle
      langsamer();
 break;
 
 case 16734885: // schneller
      SerialBT.println("schneller"); // Ausgabe auf der Seriellen schnittstelle
      schneller();
 break;

 case 16738455: // Licht
      SerialBT.println("licht"); // Ausgabe auf der Seriellen schnittstelle
      licht();
      blinken();
 break;

 case 16756815: // Hupe
      SerialBT.println("hupen");  // Ausgabe auf der Seriellen schnittstelle
      hupen();
      blinken();
 break;
 
}
}



